% interpdemo.m

% Demonstrate linear and cubic interpolation methods for interp2.
%
% Nick Kingsbury, Cambridge University, 2006.

clear all
close all

z = [0 0 0 1 0 0 0 0 1 0 1 1 1 1 0.75 0.5 0.25 0 0 0]';
zi = [1:length(z)] - 0.5;
z2d = [z z z]; % 2-D version of z , since interp2 needs 2-D input.
ri=[0:0.025:length(z)]';
rim = max(min(ri,length(z)-0.5),0.5);

% Linear interp.
zlin = interp2(z2d,2,rim+0.5,'linear');
t = [11:20:length(ri)];
pos1 = [0.06 0.3 0.9 0.4];
figure(1);
set(gcf,'position',[220    40   790   526]);
subplot('position',pos1);
plot(ri,zlin,'-g','Linewidth',2);
axis([0 20 -0.2 1.2]);
hold on; 
h1 = plot(zi,z,'ro','Linewidth',2,'Markersize',4); 
plot(ri(t),zlin(t),'bx','Linewidth',1,'Markersize',10); 
hold off

print -depsc interplin.eps

% Cubic interp.
zcub = interp2([z z z],2,rim+0.5,'cubic');
figure(2);
set(gcf,'position',[220    40   790   526]);
subplot('position',pos1);
plot(ri,zcub,'-g','Linewidth',2);
axis([0 20 -0.2 1.2]);
hold on; 
h1 = plot(zi,z,'ro','Linewidth',2,'Markersize',4); 
plot(ri(t),zcub(t),'bx','Linewidth',1,'Markersize',10); 
hold off

print -depsc interpcub.eps

% Fourier transforms of impulse responses for linear and cubic methods.
d = round(1/diff(ri(1:2)));  % Interpolation upsampling factor.
t1 = [1.5*d:5.5*d].'; % Samples from ri which define impulse response.
impresp = [zlin(t1)  zcub(t1)];
n2 = length(t1); % n2 will be odd.
n1 = (n2+1)/2; % mid-point of impresp.
freqresp = fft([impresp(n1:n2,:); zeros(2048-n2,2); impresp(1:n1-1,:)]);
for k=1:2, freqresp(:,k) = freqresp(:,k)/freqresp(1,k); end
f = [0:1024].'*(d/2048);
figure(3);
set(gcf,'position',[220    40   790   526]);
subplot('position',[0.06 0.2 0.9 0.7]);
hf = plot(f,real(freqresp(1:1025,:)),'Linewidth',2);
set(hf(1),'LineStyle','--');
grid on
axis([0 3 -0.1 1.1])
text(f(22),real(freqresp(22,1)),'Linear','Fontsize',14,'horiz','r','vert','top');
text(f(22),real(freqresp(22,2)),'Cubic','Fontsize',14,'horiz','l','vert','bot');

print -depsc interpfreq.eps

