% ph_enhance.m
% Routine called by 'Enhance' menu item in photo editor.
%
% Nick Kingsbury, Cambridge University, 2006.

switch mode
    
    case 'Init'  
        % Set up figure window for commands
        figure(2);
        set(gcf,'position',[fig2pos  450  100]);
        set(gcf,'numbertitle','off','name','Enhance Image');
        
        % Initialise denoising and sharpening edge detection thresholds.
        sc = 100;
        if exist('thresh') ~= 1, thresh = sc*[0.05 0.1 1.0]'; end
        
        % Buttons for different thresholds and filter strength.
        label = {'Denoise thr:','Sharpen thr:','Strength(%):'};
        for k = 1:3,
            ypos = 70 - 20*k; xpos = 10;
            slide(k) = uicontrol(gcf,'style','slider','sliderstep',[1 5]/100,...
                'pos',[xpos+70 ypos 120 20],'call','mode=''Slider''; ph_enhance');
            uicontrol(gcf,'style','text','str',label{k},'pos',[xpos ypos 70 20]);
            edbox(k) = uicontrol(gcf,'style','edit','pos',[xpos+190 ypos 60 20],...
                'call','mode=''Edit box''; ph_enhance');
        end
        uicontrol(gcf,'style','text','str','Adjust Thresholds','pos',[70 70 120 16]);
        
        % Other buttons: Denoise, Enhance, Revert and Close.
        denoise = uicontrol(gcf,'pos',[310 70 130 20],...
            'call','mode=''Denoise''; ph_enhance');
        sharpen = uicontrol(gcf,'pos',[310 40 130 20],...
            'call','mode=''Sharpen''; ph_enhance');
        revert = uicontrol(gcf,'str','Revert','pos',[310 10 60 20],...
            'call','mode=''Revert''; ph_enhance');
        closebtn = uicontrol(gcf,'str','Close','pos',[380 10 60 20],...
            'call','mode=''Close''; ph_enhance');
        
        % Initialise variables and set yui = xui.
        y = double(xui) + 0.5;
        iter_n = 0;
        iter_s = 0;
        set(denoise,'str',sprintf('Reduce noise, iter %d',iter_n));
        set(sharpen,'str',sprintf('Sharpen edges, iter %d',iter_s));
        for k=1:3, 
            set(slide(k),'value',thresh(k)/sc);
            set(edbox(k),'string',sprintf('%.2f',thresh(k)));
        end;
        yui=uint8(y); 
        % Calculate edge maps for both thresholds.
        [gainh,gainv] = im_edges(xui,thresh(1:2)); 
        update_y; 
        figure(2);
        
    case 'Slider'
        for k=1:3, 
            thresh(k) = sc*get(slide(k),'value');
            set(edbox(k),'string',sprintf('%.2f',thresh(k)));
        end;
        [gainh,gainv] = im_edges(xui,thresh(1:2)); 
        
    case 'Edit box'
        for k=1:3, 
            thresh(k) = sscanf(get(edbox(k),'string'),'%f');
            set(slide(k),'value',thresh(k)/sc);
            set(edbox(k),'string',sprintf('%.2f',thresh(k)));
        end;
        [gainh,gainv] = im_edges(xui,thresh(1:2)); 
        
    case 'Denoise'
        iter_n = iter_n + 1;
        set(denoise,'str',sprintf('Reduce noise, iter %d',iter_n));
        cg = thresh(3)/400;
        y = im_enhance(y,cg*gainh(:,:,1),cg*gainv(:,:,1)); 
        yui=uint8(y); 
        update_y; 
        figure(2);
        
    case 'Sharpen'
        iter_s = iter_s + 1;
        set(sharpen,'str',sprintf('Sharpen edges, iter %d',iter_s));
        cg = thresh(3)/400;
        y = im_enhance(y,cg*(gainh(:,:,2)-1),cg*(gainv(:,:,2)-1)); 
        yui=uint8(y); 
        update_y; 
        figure(2);
        
    case 'Revert'
        y = double(xui)+0.5;
        yui=uint8(y); 
        iter_n = 0;
        iter_s = 0;
        update_y; 
        figure(2);
        set(denoise,'str',sprintf('Reduce noise, iter %d',iter_n));
        set(sharpen,'str',sprintf('Sharpen edges, iter %d',iter_s));
        
    case 'Close'
        close(2); 
        newbefore = 1; 
        showimages;

    end
    
